import java.util.*;

public class Main {
	public static void main(String[] args) {
		// Get commandline arguments
		// Checks that only the file has been declared
		
		// TODO Add arguments to read files from
//		if (args.length != 1) {
//			System.out.println("Invalid arguments, you need to define the text file to read from!");
//			return;
//		}
//		
//		System.out.println(Arrays.toString(args));
		
		// Get the data from the file
		List<String> data = ReadFile.open("example.txt");
		
		// Gets a map with the total occurrences for each domain
		Map<String, Integer> domains = getDomainTotal(data);

		// Order the map by descending value of the value
		List<Map.Entry<String, Integer>> sortedDomains = sortDomainMapDesc(domains);
		
		// Format the data to the format specified
		String output = domainListToString(sortedDomains);
		
		System.out.println(output);
	}

	/**
	 * Totals up the amount of times an email domain occured in the list. 
	 * @param emails A list of emails
	 * @return A map of email domains and the total amount of times they occured in the <code>emails</code> parameter
	 */
	private static Map<String, Integer> getDomainTotal(List<String> emails) {
		// Create a map of all email address domains and counting them
		Map<String, Integer> domains = new HashMap<>();
		for (String email : emails) {
			if (!EMail.validate(email)) {
				throw new IllegalArgumentException("There was an invalid email in the file!");
			}

			String domain = EMail.getDomain(email);

			// Add email domain to the list
			if (!domains.containsKey(domain)) {
				// Creates new entry in the map if the domain didn't exist before
				domains.put(domain, 1);
			} else {
				// Updates the map with the new total for the domain
				// Gets the old total and adds 1
				Integer newTotal = domains.get(domain) + 1;

				// Update domain value with new total
				domains.replace(domain, newTotal);
			}
		}
		
		return domains;
	}

	/**
	 * Sorts the domains in descending order, from most occurrences to the least occurrences of domains.
	 * @param domains An unsorted map of domains with the amount that they occur
	 * @return Sorted list of Map Entries, in descending order of occurrences
	 */
	private static List<Map.Entry<String, Integer>> sortDomainMapDesc(Map<String, Integer> domains) {
		// Create a list from the map entries
		List<Map.Entry<String, Integer>> sortedDomains = new ArrayList<>(domains.entrySet());
		
		// Sort the list in descending order
		sortedDomains.sort(Map.Entry.comparingByValue(Comparator.reverseOrder()));
		
		return sortedDomains;
	}

	/**
	 * Creates a string in a format to display the data from a list of domains and their occurrences.
	 * @param domains A list of all the domains and their occurrences
	 * @return A string with a specific format
	 */
	private static String domainListToString(List<Map.Entry<String, Integer>> domains) {
		StringBuilder output = new StringBuilder();
		for (Map.Entry<String, Integer> entry : domains) {
			// Get the values of the entry
			String domain = entry.getKey();
			Integer occurrences = entry.getValue();
			
			output.append(domain).append(" ").append(occurrences).append("\n");
		}
		
		// Remove unnecessary \n at the end of the output
		output.deleteCharAt(output.length() - 1);
		
		return output.toString();
	}
}